import Trace::*;
import Vector::*;

`include "asim/provides/audio_pipe_types.bsh"

`define MAX_VOICES       4
`define MAX_CORES       16
`define MAX_PATH_IDS    18
`define MAX_PATH_LENGTH 8

// The path is hardwired and so should never be stored in a register.  Therefore int type rather than Bit type

typedef Bit#(32) MemAddr;
typedef Int#(TLog#(`MAX_PATH_IDS))  PathId;
//typedef Int#(16) Sample;
typedef Int#(TLog#(`MAX_VOICES)) VoiceId;
typedef Int#(8) Volume; // This is arbitrarily set to 8 bits or max val 256

//The mixer is identified as PathId 0, path end is max 
PathId mixerId = 0;
PathId endId = `MAX_CORES+1;

// Path is array of path ids
typedef Vector#(`MAX_PATH_LENGTH,  PathId) CorePath;
CorePath emptyCore = replicate(endId);

typedef struct
{
  VoiceId   voice;
  MemAddr   startAddr;
  CorePath  route;
  Volume    outVol;
} FullPath deriving (Bits, Eq);

typedef Maybe#(AudioProcessorUnit) AudioPipeUnit; // deriving (Bits, Eq);

typedef struct
{
  VoiceId       voice;
  AudioPipeUnit data;
} AudioStream deriving (Bits, Eq);  
